// day.C  Version 2.20  Copyleft (c) 杉本典夫
// Time-stamp: <2022-11-17 17:04:24 tac>
//
// 年月日・日数相互変換用共通関数
// 年月日は「[<元号記号1桁>]<年4または2桁>[{/|-}]<月2桁>[{/|-}]<日2桁>」という
// 形式の文字列で、西暦の場合は元号記号なし年４桁、和暦の場合は元号記号１桁
// 年２桁となる。
//	元号記号　　明治:m,M　大正:t,T　昭和:s,S　平成:h,H
// 元号記号なし年２桁の時は、現在年の下２桁未満は西暦20??年、下２桁以上は
// 西暦19??年と解釈する。
// 日数変換関数day()は、通常は２つの年月日の間の間隔を計算するのに用いるが、
// 色々な形式の年月日をday()関数によって一旦日数に変換し、それをdate()関数
// によって年月日に戻すことによって形式を統一することができる。
//
// ライブラリ：libfun.so.2.2
// コンパイル法：g++ -O -fPIC -c day.C

#include <fun.H>                // 共通関数用ヘッダファイル

using namespace fun;            // fun名前空間使用(名前空間std含む)

const string& fun::date(string& dstr, long d)
// 日数を西暦年月日に変換して返す
{
  dstr.erase();
  if (d < 1) return dstr;       // 日数不適
  long c = (4 * d - 1) / 146097;
  d = ((4 * d - 1) % 146097) / 4;
  long y = (4 * d + 3) / 1461;
  d = ((4 * d + 3) % 1461) / 4 + 1;
  long m = (5 * d - 3) / 153;
  d = ((5 * d - 3) % 153) / 5 + 1;
  if (m < 10) m += 3;
  else { m -= 9;  y++; }
  ostringstream ostr;
  ostr.setf(ios::right, ios::adjustfield);
  ostr.fill('0');
  ostr << setw(4) << c * 100 + y;
  ostr << setw(2) << m;
  ostr << setw(2) << d;
  dstr = ostr.str();
  return dstr;
}

long fun::day(const string& dstr)
// 年月日を日数に変換して返す
{
  if (dstr.empty()) return -1;  // 年月日文字列不適
  string tstr(dstr);
  char id = tolower(tstr[0]);
  long base = 0;
  switch (id)
  {
  case 'm' : tstr.erase(0, 1);  base = 1867; // 明治
    break;
  case 't' : tstr.erase(0, 1);  base = 1911; // 大正
    break;
  case 's' : tstr.erase(0, 1);  base = 1925; // 昭和
    break;
  case 'h' : tstr.erase(0, 1);  base = 1988; // 平成
    break;
  default : id = '\0';  base = 0; // 西暦
  }
  long c = 0, y = 0, m = 0, d = 0;
  if (tstr.find('/') != string::npos) // 年/月/日形式
  { sscanf(tstr.c_str(), "%ld/%ld/%ld", &y, &m, &d); }
  else if (tstr.find('-') != string::npos) // 年-月-日形式
  { sscanf(tstr.c_str(), "%ld-%ld-%ld", &y, &m, &d); }
  else if (tstr.find('.') != string::npos) // 年.月.日形式
  { sscanf(tstr.c_str(), "%ld.%ld.%ld", &y, &m, &d); }
  else                          // 年月日形式
  {
    if (tstr.size() > 6) sscanf(tstr.c_str(), "%4ld%2ld%2ld", &y, &m, &d);
    else sscanf(tstr.c_str(), "%2ld%2ld%2ld", &y, &m, &d);
  }
  if (y < 0 || m < 1 || d < 0) return -3; // 文字列不適
  if (d < 1) d = 1;             // 日がない時は１日とする
  if (id != '\0')
  { y += base; }
  else if (y < 100)             // 西暦下２桁と解釈
  {
    long t = time((long *)0);   // 現在時間取得
    char tmp[8];
    strftime(tmp, 6, "%C", localtime(&t)); // 現在の世紀(西暦上２桁)
    char* ptr = 0;
    c = strtol(tmp, &ptr, 10);
    if (c < 1) c = 20;
    y += c * 100;
    strftime(tmp, 6, "%Y", localtime(&t)); // 現在の西暦年
    c = strtol(tmp, &ptr, 10);
    if (c < 1) c = 2000;
    if (y > c) y -= 100;        // 西暦19??と解釈
  }
  if (m > 2) m -= 3;
  else { m += 9;  y--; }
  c = y / 100;
  y = y % 100;
  return (146097 * c) / 4 + (1461 * y) / 4 + (153 * m + 2) / 5 + d;
}
