// fun.H  Version 2.2  Copleft (c) 杉本典夫
// Time-stamp: <2023-06-17 09:46:57 tac>
//
// 各種の自作プログラムで共通に使用するヘッダファイルや定数、自作の一般的な
// ライブラリィ関数のプロトタイプ宣言を行う共通関数用ヘッダファイル。
//
// 関連ライブラリ:libfun.so.2.2

#ifndef _FUN                    // 重複インクルードを防ぐため
#define _FUN

/* Cのヘッダファイルインクルード */
#include <cstdio>               // 標準入出力用ヘッダファイル
#include <cstdlib>              // C標準ライブラリ関数用ヘッダファイル
#include <cctype>               // 文字検査用ヘッダフイル
#include <cstring>              // 文字列処理関数用ヘッダフイル
#include <climits>              // 限界値用ヘッダファイル
#include <cfloat>               // 浮動小数点の限界値用ヘッダファイル
#include <cmath>                // 数学関数用ヘッダファイル
#include <unistd.h>             // UNIXライブラリ関数用ヘッダファイル
#include <fcntl.h>              // 低水準入出力用ヘッダファイル
#include <sys/stat.h>           // ファイルモード用ヘッダファイル
#include <regex.h>              // 正規表現用ヘッダファイル
#include <iconv.h>              // 日本語コード変換関数用ヘッダファイル
#include <boost/multiprecision/cpp_dec_float.hpp> // 多倍精度ライブラリ用ヘッダファイル

/* C++標準関数のヘッダファイルインクルード */
#include <iostream>             // IOStream用ヘッダファイル
#include <fstream>              // ファイル入出力用ヘッダファイル
#include <sstream>              // 文字列ストリーム用ヘッダファイル
#include <string>               // 文字列クラス用ヘッダファイル
#include <vector>               // ベクトルクラス用ヘッダファイル
#include <list>                 // リストクラス用ヘッダファイル
#include <iomanip>              // 入出力マニピュレータ用ヘッダファイル
#include <algorithm>            // 各種アルゴリズム用ヘッダファイル
#include <set>                  // set用ヘッダファイル
#include <map>                  // map用ヘッダファイル
#include <limits>               // 限界値用ヘッダファイル
#include <locale>               // ロケール用ヘッダファイル
#include <stdexcept>            // 標準例外用ヘッダファイル

namespace fun                   // 共通関数用名前空間fun定義
{
  using namespace std;          // C++標準ライブラリの名前空間を合成する
  using namespace boost::multiprecision; // 多倍精度ライブラリ名前空間を合成する

/* 変数型定義 */
  using uchar = unsigned char;  // 文字処理関数のため

  using uint = unsigned int;    // BCBの定義と合わせるため
  using uint8 = int8_t;         // BCBの定義と合わせるため
  using uint16 = int16_t;       // BCBの定義と合わせるため
  using uint32 = int32_t;       // BCBの定義と合わせるため
  using llint = long long int;  // 長精度整数の短名定義
  using ullint = unsigned long long int; // 長精度符号無し整数の短名定義

  using ldbl = long double;     // 4倍精度実数の短名定義

  using mpf50 = cpp_dec_float_50; // 多倍精度実数50桁の短名定義
  using mpf100 = cpp_dec_float_100; // 多倍精度実数100桁の短名定義

  using strsize_t = string::size_type; // 文字列のサイズ型の短名定義

  using lstr = list<string>;               // 文字列リストの短名定義
  using lstr_itr = list<string>::iterator; // 反復子の短名定義
  using lstr_citr = list<string>::const_iterator; // 反復子の短名定義

  using vecstr = vector<string>;  // 文字列ベクトルの短名定義
  using vecstr_itr = vector<string>::iterator; // 反復子の短名定義
  using vecstr_citr = vector<string>::const_iterator; // 反復子の短名定義

  using vecbool = vector<bool>;           // boolベクトルの短名定義
  using vecint = vector<int>;             // 整数ベクトルの短名定義
  using vecllint = vector<long long int>; // 長精度整数ベクトルの短名定義
  using vecdbl = vector<double>;       // 倍精度実数ベクトルの短名定義
  using vecldbl = vector<long double>; // 4倍精度実数ベクトルの短名定義

  using pairint = pair<int, int>; // 整数ペアの短名定義
  using pairdbl = pair<double, double>; // 倍精度実数ペアの短名定義
  using pairldbl = pair<long double, long double>; // 4倍精度実数ペアの短名定義

/* 定数定義 */
  constexpr uint LENbuf = 4096; // バッファサイズ
  constexpr uint LENcrt = 80;   // CRT１行あたりの文字数

  constexpr uint CHARascii = 0x0001;   //7ビットASCII文字指定
  constexpr uint CHARhkana = 0x0002;   // 半角カタカナ文字指定
  constexpr uint CHARkanji = 0x0004;   // 漢字文字指定
  constexpr uint CHARexkanji = 0x0008; // 拡張漢字文字指定
  constexpr uint CHARetc = 0x0080;     // その他の文字指定
  constexpr uint CHARall = 0x00FF;     // 全ての文字指定
  constexpr uint CHARwide = 0x0100;    // ワイド文字の出力桁数指定

  constexpr char CHResc = '\\';   // エスケープ文字「\」
  constexpr char CHRopt = '-';    // オプション指定文字
  constexpr char CHRsquot = '\''; // 単引用文字
  constexpr char CHRdquot = '"';  // 復引用文字

  constexpr char OPTdelm = ',';  // 複数オプションの区切り文字
  constexpr char OPTdev = '=';   // オプションの種類と内容分離文字
  constexpr char OPTcom = '#';   // オプションのコメント指定文字

  constexpr char ENVdelm = ';';  // 環境変数の区切り文字
  constexpr char ENVdev = ':';   // 環境変数の種類と内容分離文字

/* 列挙型定義 */
  enum codetype                 // 日本語コードの種類
  {
    codenon,                    // コード指定なし
    jis,                        // JIS
    sjis,                       // Shift-JIS
    euc,                        // EUC-JP
    utf8,                       // UTF-8
    codeetc                     // その他
  };

  enum esctype              // JIS漢字コードエスケープシーケンスの種類
  {
    escnon,                     // エスケープシーケンスではない
    kin,                        // 漢字IN
    kout,                       // 漢字OUT
    escetc                      // その他
  };

class help_except : public runtime_error // ヘルプ表示用例外クラス
{
public:
  explicit help_except(const string& what_arg) : runtime_error(what_arg)
  // コンストラクタ
  {}
  ~help_except() throw()        // デストラクタ
  {}
};

class option_error : public runtime_error // オプション指定エラー用例外クラス
{
public:
  explicit option_error(const string& what_arg) : runtime_error(what_arg)
  // コンストラクタ
  {}
  ~option_error() throw()       // デストラクタ
  {}
};

/* 特殊関数群(インライン関数・マクロ) */
inline void beep()              // ベルを鳴らす
{ cout << "\x07"; }

inline uint pow2(int n)         // 2のベキ乗関数
{ return 1 << n; }

/* バイト処理関数群(テンプレート・インライン関数) */
template <class T> T* revcopy(T* dest, T* source, size_t size)
// 要素の並びを逆順にしながらコピーする
{
  if (dest == 0 || source == 0) return nullptr;
  T *dptr = dest, *sptr = source;
  for (size_t i = 0; i < size; ++i, ++dptr) *dptr = *(sptr + size - 1 - i);
  return dest;                  // destを返す
}

inline void* bytecopy(void* dest, void* source, size_t size)
// バイトコピー(Linuxの場合はIntel型の逆順バイトコピー)
{ return (void*)revcopy((uint8*)dest, (uint8*)source, size); }

/* 日本語コード処理関数群(ソースファイル:jcode.C) */
extern int iseuchkana(uchar c);   // EUC半角カタカナ1バイト目の検査
extern int iseuckanji(uchar c);   // EUC漢字1/2バイト目の検査
extern int iseucexkanji(uchar c); // EUC拡張漢字1バイト目の検査
extern int iseuczspace(uchar chigh, uchar clow); // EUC全角スペースの検査
extern int iseuczkana(uchar chigh, uchar clow); // EUC全角カタカナの検査

extern int isjishkana(uchar c); // JIS半角カタカナの検査

extern int issjishkana(uchar c);  // Shift-JIS半角カタカナの検査
extern int issjiskanji(uchar c);  // Shift-JIS漢字1バイト目の検査
extern int issjiskanji2(uchar c); // Shift-JIS漢字2バイト目の検査
extern int issjiszspace(uchar chigh, uchar clow); // Shift-JIS全角スペースの検査
extern int issjiszkana(uchar chigh, uchar clow); // Shift-JIS全角カタカナの検査

extern int nutf8(uchar c); // UTF-8の1バイト目からコードのバイト数を返す
extern int isutf8mcode(uchar c);  // UTF-8マルチバイトコード1バイト目の検査
extern int isutf8mcode2(uchar c); // UTF-8マルチバイトコード2-4バイト目の検査
extern int isutf8hkana(uchar c1, uchar c2, uchar c3); // UTF-8半角カタカナの検査
extern int isutf8zspace(uchar c1, uchar c2, uchar c3); // UTF-8全角スペースの検査
extern int isutf8zkana(uchar c1, uchar c2, uchar c3); // UTF-8全角カタカナの検査

extern codetype langcode();     // 現在の言語コードタイプを返す

extern int ncode(uchar c, codetype cdtype = codenon);
// 1コードのバイト数を返す(汎用)
extern int ismcode(uchar c, codetype cdtype = codenon);
// マルチバイトコードの検査(汎用)
extern int ishkana(const char* str, codetype cdtype = codenon);
// 半角カタカナの検査(汎用)
extern int iskanji(const char* str, codetype cdtype = codenon);
// 漢字の検査(汎用)
extern int iszspace(const char* str, codetype cdtype = codenon);
// 全角スペースの検査(汎用)
extern int iszkana(const char* str, codetype cdtype = codenon);
// 全角カタカナの検査(汎用)

extern wchar_t ctow(unsigned char high, unsigned char low);
// ２つの１バイト文字を１つのワイド文字に変換して返す
extern wchar_t wtoc(unsigned char& high, unsigned char& low, wchar_t w);
// １つのワイド文字を２つの１バイト文字に変換する

extern wchar_t euchantozen(uchar& high, uchar& low);
// EUC半角ANKを全角漢字にして返す
extern wchar_t euczentohan(uchar& high, uchar& low);
// EUC全角漢字を半角ANKにして返す

/* 日本語文字列処理関数群(ソースファイル:jstr.C) */
extern esctype jisescseq(const string& str);
// JIS漢字コードエスケープシーケンスの検査

int cdconv(const char* inbuf, size_t insize, codetype incode, char* outbuf, size_t outsize, codetype outcode);
// inbuf中の日本語コードを変換してoutbufに格納して返す

extern int streuctojis(string& str); // 文字列中のEUCコードをJISコードに変換する
extern int streuctosjis(string& str);
// 文字列中のEUCコードをShift-JISコードに変換する
extern int streuctoutf8(string& str);
// 文字列中のEUCコードをUTF-8コードに変換する

extern int strjistoeuc(string& str);
// 文字列中のJISコードをEUCコードに変換する
extern int strsjistoeuc(string& str);
// 文字列中のShift-JISコードをEUCコードに変換する
extern int strutf8toeuc(string& str);
// 文字列中のUTF-8コードをEUCコードに変換する

extern int strutf8tojis(string& str);
// 文字列中のUTF-8コードをJISコードに変換する
extern int strutf8tosjis(string& str);
// 文字列中のUTF-8コードをShift-JISコードに変換する

extern int strjistoutf8(string& str);
// 文字列中のJISコードをUTF-8コードに変換する
extern int strsjistoutf8(string& str);
// 文字列中のShift-JISコードをUTF-8コードに変換する

extern int strhantozen(string& str, char mode, codetype cdtype = codenon);
// 文字列中の半角ANKを全角ANKに変換する(汎用)
extern int strzentohan(string& str, char mode, codetype cdtype = codenon);
// 文字列中の全角ANKを半角ANKに変換する(汎用)

extern int strtrm(string& str, codetype cdtype = codenon);
// 文字列の前後の空白を削除する(全角スペース含む、汎用)

extern int strclen(const string& str, uint chartype, codetype cdtype = codenon);
// 文字列中の指定された文字の数を返す(汎用)

/* 文字列処理関数群(ソースファイル:str.C) */
extern int strchomp(string& str); // 文字列の末尾の改行記号を削除する(汎用)

extern int strunquot(string& str, codetype cdtype = codenon);
// 文字列を囲む引用文字を削除する(汎用)

extern int strlower(string& str, codetype cdtype = codenon);
// 文字列中のアルファベットを小文字に変換する(汎用)
extern int strupper(string& str, codetype cdtype = codenon);
// 文字列中のアルファベットを大文字に変換する(汎用)

extern int strreplace(string& str, const string& targstr, const string& reprstr, bool global = false, codetype cdtype = codenon);
// 文字列中の部分文字列を置換する(汎用)

extern int strsplit(lstr& dstrlst, const string& sstr, char delmc, codetype cdtype = codenon);
// 文字列を区切り文字で分割して文字列リストに追加する(汎用)
extern int strsplit(vecstr& dstrvec, const string& sstr, char delmc, codetype cdtype = codenon);
// 文字列を区切り文字で分割して文字列ベクトルに追加する(汎用)

extern strsize_t getsubstr(string& substr, const string& sstr, char delmc, codetype cdtype = codenon);
// 文字列から区切り文字までの部分文字列を取り出す(汎用)
extern char* getsubstr(string& substr, const char* sstr, char delmc, codetype cdtype = codenon);
// 文字配列から区切り文字までの部分文字列を取り出す(EUC/UTF-8/Shift-JISのみ)

extern int sstrtostr(string& str, codetype cdtype = codenon);
// 特殊文字列を文字列に変換する(汎用)
extern int strtosstr(string& str); // 文字列を特殊文字列に変換する(汎用)

extern int strpmatch(const string& str, const string& pattern, codetype cdtype = codenon);
// 文字列中にパターン(正規表現を含む)があるかどうかをチェックする(汎用)
extern int strregrep(string& sline, const string& pattern, const string& reprstr, bool global = false, codetype cdtype = codenon);
// 正規表現を含む部分文字列を置換する(汎用)

/* ファイル入出力操作関数群(ソースファイル:getfile.C) */
extern int getsline(string& sline, istream& fin);
// 入力ストリームから文字列に１行読み込む
extern int getfile(uint8** buf, const char* fname);
// バイト配列にファイル全体を読み込む
extern int putfile(const uint8* buf, size_t bufsize, const char* fname);
// バイト配列をファイルに書き込む

extern string getname(const char* c_str); // ファイル基本名を取り出す

extern bool ofopen(ofstream& fout, const char* fname);
// 出力用ファイルを開いてファイルストリームを返す(fname->fname.bakにする)

/* 各種出力処理関数群(ソースファイル:put.C) */
extern int disperr(const string& pname, const string& msg1 = "", const string& msg2 = "");
// エラーメッセージを標準エラー出力に出力する

extern void putline(ostream& fout, int n, char linec = '-');
// 区切り線を出力する

/* オプション指定処理関数群(ソースファイル:option.C) */
extern int getopt(string opt[], const string& str, char devc = OPTdev, char comc = OPTcom);
// 文字列からオプションの種類と指定内容を取り出し文字列配列に格納する

extern int badopt(const string& pname, const string& opstr, const vecstr& usage, double low = DBL_MAX, double up = DBL_MAX);
// オプションの指定法が間違っている時の処理
extern int nonopt(const string& pname, const string& opstr, const vecstr& usage);
// 指定されたオプションがなかった場合の処理

extern int help(const string& pname, const vecstr& usage, const string& man = "");
// オンラインマニュアルの表示

/* オプション読み込み関数群(テンプレート) */
template <class T>
int fgetopt(T& opt, const string& optfile, bool fcheck = false)
// オプション指定をファイルから読み込む
{
  if (optfile.empty())
  {
    if (fcheck) return disperr(opt.pname(), "オプション指定ファイルが指定されていません");
    return 0;
  }
  ifstream fin(optfile.c_str());
  if (!fin)
  {
    if (fcheck) return disperr(opt.pname(), "オプション指定ファイルがありません→", optfile);
    return 0;
  }
  string tstr;
  while (getsline(tstr, fin) != EOF) opt.set(tstr);
  return 0;
}

template <class T> int envgetopt(T& opt) // オプション指定を環境変数から読み込む
{
  const char* envptr = getenv((opt.pname()).c_str());
  if (envptr == nullptr) return 0; // 環境変数が指定されていない
  string tstr;
  while ((envptr = getsubstr(tstr, envptr, ENVdelm)) != nullptr)
  {
    opt.set(tstr, ENVdev);
    if (*envptr != '\0') ++envptr;
  }
  return 0;
}

template <class T> int getoptdef(T& opt)
// 省略時オプション指定を環境変数とoptファイルから読み込む
{
  envgetopt(opt);              // オプション指定を環境変数から読み込む
  string optfile(opt.path() + opt.pname() + ".opt");
  return fgetopt(opt, optfile); // オプション指定を$PATH/<pname>.optファイルから読み込む
}

/* 数値処理関数群(テンプレート) */
template <class T> int sgn(T x) // 符合関数
{
  if (fabs(x) < numeric_limits<T>::min() / numeric_limits<T>::epsilon()) return 0;
  // 最小値のEPSILON倍未満は0にする
  return (x > (T)0.0) ? 1 : -1;
}

template <class T> int sgndiff(T x, T y) // 差の符合関数
{
  T xyabs = fabs(x) + fabs(y);
  if (sgn(xyabs) == 0) return 0; // |x|+|y|≒0なら0にする
  T d = x - y;
  if (fabs(d) < numeric_limits<T>::epsilon() * xyabs) return 0;
  // 差の絶対値がxyabs×epsilon未満は0にする
  return (d > (T)0.0) ? 1 : -1;
}

template <class T> int getrange(T& low, T& hight, const string& str)
// 文字列から数値範囲を取り出す
{
  if (str.empty()) return 0;    // 指定なし
  double ld = 0.0, hd = 0.0;
  strsize_t idx = 0;
  if (str[0] == '-')
  {
    string substr(str.substr(1));
    if (substr.empty()) return 0; // 指定なし
    if (substr[0] == '-')
    {
      sscanf(substr.c_str(), "%lf", &hd);
      hight = (T)hd;
      return -0x00000002;       // --h:上限(負)のみ指定
    }
    if ((idx = substr.rfind("-")) == string::npos)
    {
      sscanf(substr.c_str(), "%lf", &hd);
      hight = (T)hd;
      return 0x00000002;        // -h:上限(正)のみ指定
    }
    if (substr.substr(idx) == "-")
    {
      sscanf(str.c_str(), "%lf", &ld);
      low = (T)ld;
      return -0x00000001;       // -l-:下限(負)のみ指定
    }
    sscanf(str.c_str(), "%lf-%lf", &ld, &hd);
    low = (T)ld;  hight = (T)hd;
    return -0x00000003;         // -l-h:下限(負)・上限指定
  }
  if ((idx = str.rfind("-")) == string::npos)
  {
    sscanf(str.c_str(), "%lf", &ld);
    low = (T)ld;
    hight = low;
    return 0x00000003;          // l:下限=上限(正)指定
  }
  if (str.substr(idx) == "-")
  {
    sscanf(str.c_str(), "%lf", &ld);
    low = (T)ld;
    return 0x00000001;          // l-:下限(正)のみ指定
  }
  sscanf(str.c_str(), "%lf-%lf", &ld, &hd);
  low = (T)ld;  hight = (T)hd;
  return 0x00000003;            // l-h:下限・上限(正)指定
}

template <class T>
int getrangelst(list<pair<T, T>>& rangelst, const string& sstr, char delmc)
// 文字列から数値範囲指定を読み込みリストに追加する
{
  lstr rstrlst;
  if (strsplit(rstrlst, sstr, delmc) < 1) return 0; // 有効な範囲指定なし
  size_t size0 = rangelst.size();
  for (lstr_itr pos = rstrlst.begin(); pos != rstrlst.end(); ++pos)
  {
    pair<T, T> range;
    range.first = - numeric_limits<T>::max();
    range.second = numeric_limits<T>::max();
    sstrtostr(*pos);
    if (getrange(range.first, range.second, *pos) == 0) return 0;
    rangelst.push_back(range);
  }
  return rangelst.size() - size0; // 読み込んだ範囲指定数を返す
}

/* ビット操作関数群(ソースファイル:packcode.C) */
extern int packcode(uint8* dest, const uint8* source, size_t ns, int nbits);
// 可変長コード列をMBSFirstモードでバイト列にパックする
extern int packvcode(uint8* dest, uint c, int nbits, int cmd);
// 可変長コードをMBSFirstモードでバイトにパックする
extern int unpackcode(uint8* dest, const uint8* source, size_t ns, int nbits);
// MBSFirstモードでパックされた可変長コード列をバイト列にアンパックする
extern int unpackvcode(const uint8* source, size_t ns, int nbits, int cmd);
// MBSFirstモードでパックされた可変長コードをアンパックする

/* 日付操作関数群(ソースファイル:day.C) */
extern const string& date(string& dstr, long d);
// 日数を西暦年月日に変換して返す

extern long day(const string& dstr); // 年月日を日数に変換して返す

} // 名前空間fun終わり

#endif
