// jcode.C  Version 2.20  Copyleft (c) 杉本典夫
// Time-stamp: <2022-11-17 17:04:47 tac>
//
// 日本語コードの各種処理を行う関数群
//
// ライブラリ：libfun.so.2.2
// コンパイル法：g++ -O -fPIC -c jcode.C

#include <fun.H>                // 共通関数用ヘッダファイル

using namespace fun;            // fun名前空間使用(名前空間std含む)

int fun::iseuchkana(uchar c)
// EUC半角カタカナ1バイト目の検査
{
  return (c == 0x8E) ? 1 : 0;
}

int fun::iseuckanji(uchar c)
// EUC漢字1/2バイト目の検査
{
  return (0xA1 <= c && c <= 0xFE) ? 1 : 0;
}

int fun::iseucexkanji(uchar c)
// EUC拡張漢字1バイト目の検査
{
  return (c == 0x8F) ? 1 : 0;
}

int fun::iseuczspace(uchar chigh, uchar clow)
// EUC全角スペースの検査
{
  return (chigh == 0xA1 && clow == 0xA1) ? 1 : 0;
}

int fun::iseuczkana(uchar chigh, uchar clow)
// EUC全角カタカナの検査
{
  if (chigh == 0xA5)
  {
    if (0xA1 <= clow && clow <= 0xF6) return 1; // カタカナ
  }
  else if (chigh == 0xA1)
  {
    if (clow == 0xA2 || clow == 0xA3) return 1; // 句読点中点(、。)
    if (clow == 0xA6) return 1;                 // 中点(・)
    if (clow == 0xAB || clow == 0xAC) return 1; // 濁点と半濁点
    if (clow == 0xBC) return 1;                 // 長音(ー)
    if (clow == 0xD6 || clow == 0xD7) return 1; // 括弧(「」)
  }
  return 0;                     // カタカナ以外
}

int fun::isjishkana(uchar c)
// JIS半角カタカナの検査
{ return issjishkana(c); }

int fun::issjishkana(uchar c)
// Shift-JIS半角カタカナの検査
{
  return (0xA1 <= c && c <= 0xDF) ? 1 : 0;
}

int fun::issjiskanji(uchar c)
// Shift-JIS漢字1バイト目の検査
{
  return ((0x81 <= c && c <= 0x9F) || (0xE0 <= c && c <= 0xFC)) ? 1 : 0;
}

int fun::issjiskanji2(uchar c)
// Shift-JIS漢字2バイト目の検査
{
  return ((0x40 <= c && c <= 0x7E) || (0x80 <= c && c <= 0xFC)) ? 1 : 0;
}

int fun::issjiszspace(uchar chigh, uchar clow)
// Shift-JIS全角スペースの検査
{
  return (chigh == 0x81 && clow == 0x40) ? 1 : 0;
}

int fun::issjiszkana(uchar chigh, uchar clow)
// Shift-JIS全角カタカナの検査
{
  if (chigh == 0x83)
  {
    if (0x40 <= clow && clow <= 0x96) return 1; // カタカナ
  }
  else if (chigh == 0x81)
  {
    if (clow == 0x41 || clow == 0x42) return 1; // 句読点中点(、。)
    if (clow == 0x45) return 1;                 // 中点(・)
    if (clow == 0x4A || clow == 0x4B) return 1; // 濁点と半濁点
    if (clow == 0x5B) return 1;                 // 長音(ー)
    if (clow == 0x75 || clow == 0x76) return 1; // 括弧(「」)
  }
  return 0;                     // カタカナ以外
}

int fun::nutf8(uchar c)
// UTF-8の1バイト目からコードのバイト数を返す
{
  if (0xC0 <= c && c <= 0xDF) return 2; // 2バイトコード
  if (0xE0 <= c && c <= 0xEF) return 3; // 3バイトコード
  if (0xF0 <= c && c <= 0xF7) return 4; // 4バイトコード
  return 1;                             // 1バイトコード
}

int fun::isutf8mcode(uchar c)
// UTF-8マルチバイトコード1バイト目の検査
{
  return (0xC0 <= c && c <= 0xF7) ? 1 : 0;
}

int fun::isutf8mcode2(uchar c)
// UTF-8マルチバイトコード2-4バイト目の検査
{
  return (0x80 <= c && c <= 0xBF) ? 1 : 0;
}

int fun::isutf8hkana(uchar c1, uchar c2, uchar c3)
// UTF-8半角カタカナの検査
{
  if (c1 != 0xEF) return 0;     // カタカナ以外
  if (c2 == 0xBD)
  {
    if (0xA1 <= c3 && c3 <= 0xBF) return 1; // カタカナ
  }
  else if (c2 == 0xBE)
  {
    if (0x80 <= c3 && c3 <= 0x9F) return 1; // カタカナ
  }
  return 0;                     // カタカナ以外
}

int fun::isutf8zspace(uchar c1, uchar c2, uchar c3)
// UTF-8全角スペースの検査
{
  return (c1 == 0xE3 && c2 == 0x80 && c3 == 0x80) ? 1 : 0;
}

int fun::isutf8zkana(uchar c1, uchar c2, uchar c3)
// UTF-8全角カタカナの検査
{
  if (c1 != 0xE3) return 0;     // カタカナ以外
  if (c2 == 0x82)
  {
    if (0xA1 <= c3 && c3 <= 0xBF) return 1; // カタカナ
    if (c3 == 0x9B || c3 == 0x9C) return 1; // 濁点と半濁点
  }
  else if (c2 == 0x83)
  {
    if (0x80 <= c3 && c3 <= 0xB6) return 1; // カタカナ
    if (c3 == 0xBB || c3 == 0xBC) return 1; // 中点(・)と長音(ー)
  }
  else if (c2 == 0x80)
  {
    if (c3 == 0x81 || c3 == 0x82) return 1; // 句読点(、。)
    if (c3 == 0x8C || c3 == 0x8D) return 1; // 括弧(「」)
  }
  return 0;                     // カタカナ以外
}

codetype fun::langcode()
// 現在の言語コードタイプを返す
{
  string lang = getenv("LANG"); // 環境変数LANGの内容を取得
  if (lang == "ja_JP.UTF-8" || lang == "ja_JP.utf8") return utf8;
  if (lang == "ja_JP.eucJP" || lang == "ja_JP.eucjp" || lang == "ja_JP.EUC-JP" || lang == "ja_JP.ujis") return euc;
  if (lang == "ja_JP.SJIS" || lang == "ja_JP.sjis") return sjis;
  return codeetc;
}

int fun::ncode(uchar c, codetype cdtype)
// 1コードのバイト数を返す
{
  if (cdtype == codenon) cdtype = langcode(); // コードが指定されていない時
  switch (cdtype)
  {
  case euc : if (iseuckanji(c) || iseuchkana(c)) return 2; // 2バイトコード
    if (iseucexkanji(c)) return 3;        // 3バイトコード
    break;
  case sjis : if (issjiskanji(c)) return 2;
    break;
  case utf8 : return nutf8(c);
  }
  return 1;                               // 1バイトコード
}

int fun::ismcode(uchar c, codetype cdtype)
// マルチバイトコードの検査(汎用)
{
  if (cdtype == codenon) cdtype = langcode(); // コードが指定されていない時
  switch (cdtype)
  {
  case euc : return (iseuckanji(c) || iseuchkana(c) || iseucexkanji(c));
  case sjis : return issjiskanji(c);
  case utf8 : return isutf8mcode(c);
  }
  return 0;                     // その他は1バイトコードにする
}

int fun::ishkana(const char* str, codetype cdtype)
// 半角カタカナの検査(汎用)
{
  if (str == nullptr) return 0;
  if (cdtype == codenon) cdtype = langcode(); // コードが指定されていない時
  switch (cdtype)
  {
  case euc : return iseuchkana(uchar(*str));
  case sjis : return issjishkana(uchar(*str));
  case utf8 : return isutf8hkana(uchar(str[0]), uchar(str[1]), uchar(str[2]));
  }
  return 0;                     // その他は半角カタカナ以外にする
}

int fun::iskanji(const char* str, codetype cdtype)
// 漢字の検査(汎用)
{
  if (str == nullptr) return 0;
  if (cdtype == codenon) cdtype = langcode(); // コードが指定されていない時
  uchar c1 = uchar(*str), c2 = uchar(str[1]), c3 = uchar(str[2]);
  switch (cdtype)
  {
  case euc : if (iseuckanji(c1) && iseuckanji(c2)) return 1;
    break;
  case sjis : if (issjiskanji(c1) && issjiskanji2(c2)) return 1;
    break;
  case utf8 :
    if (isutf8mcode(c1) && nutf8(c1) < 4)
    {
      if (!isutf8hkana(c1, c2, c3)) return 1; // 半角カタカナ以外
    }
    break;
  }
  return 0;                     // その他は半角カタカナ以外にする
}

int fun::iszspace(const char* str, codetype cdtype)
// 全角スペースの検査(汎用)
{
  if (str == nullptr) return 0;
  if (cdtype == codenon) cdtype = langcode(); // コードが指定されていない時
  uchar c1 = uchar(*str), c2 = uchar(str[1]), c3 = uchar(str[2]);
  switch (cdtype)
  {
  case euc : return iseuczspace(c1, c2);
  case sjis : return issjiszspace(c1, c2);
  case utf8 : return isutf8zspace(c1, c2, c3);
  }
  return 0;                     // その他は全角スペースの以外にする
}

int fun::iszkana(const char* str, codetype cdtype)
// 全角カタカナの検査(汎用)
{
  if (str == nullptr) return 0;
  if (cdtype == codenon) cdtype = langcode(); // コードが指定されていない時
  uchar c1 = uchar(*str), c2 = uchar(str[1]), c3 = uchar(str[2]);
  switch (cdtype)
  {
  case euc : return iseuczkana(c1, c2);
  case sjis : return issjiszkana(c1, c2);
  case utf8 : return isutf8zkana(c1, c2, c3);
  }
  return 0;                     // その他は全角スペースの以外にする
}

wchar_t fun::ctow(unsigned char high, unsigned char low)
// 2つの1バイト文字を1つのワイド文字に変換して返す
{
  wchar_t w1 = high, w2 = low;
  return ((w1 << 8) & 0xFF00 | (w2 & 0x00FF));
}

wchar_t fun::wtoc(unsigned char& high, unsigned char& low, wchar_t w)
// 1つのワイド文字を2つの1バイト文字に変換する
{
  high = (unsigned char)((w >> 8) & 0x00FF);
  low = (unsigned char)(w & 0x00FF);
  return w;                     // 変換前ワイド文字も返す
}

wchar_t fun::euchantozen(uchar& high, uchar& low)
// EUC半角ANKを全角漢字にして返す
{
  if (high == 0x00)             // 半角数字・ローマ字・記号
  {
    if (0x30 <= low && low <= 0x39) // 0-9
    { high = 0xA3;  low = 0xB0 + low - 0x30; }
    else if (0x41 <= low && low <= 0x5A) // A-Z
    { high = 0xA3;  low = 0xC1 + low - 0x41; }
    else if (0x61 <= low && low <= 0x7A) // a-z
    { high = 0xA3;  low = 0xE1 + low - 0x61; }
    else
    {
      switch (low)
      {
      case 0x20 : high = 0xA1;  low = 0xA1; // ' '
        break;
      case 0x21 : high = 0xA1;  low = 0xAA; // '!'
        break;
      case 0x22 : high = 0xA1;  low = 0xC9; // '"'
        break;
      case 0x23 : high = 0xA1;  low = 0xF4; // '#'
        break;
      case 0x24 : high = 0xA1;  low = 0xF0; // '$'
        break;
      case 0x25 : high = 0xA1;  low = 0xF3; // '%'
        break;
      case 0x26 : high = 0xA1;  low = 0xF5; // '&'
        break;
      case 0x27 : high = 0xA1;  low = 0xC7; // '''
        break;
      case 0x28 : high = 0xA1;  low = 0xCA; // '('
        break;
      case 0x29 : high = 0xA1;  low = 0xCB; // ')'
        break;
      case 0x2A : high = 0xA1;  low = 0xF6; // '*'
        break;
      case 0x2B : high = 0xA1;  low = 0xDC; // '+'
        break;
      case 0x2C : high = 0xA1;  low = 0xA4; // ','
        break;
      case 0x2D : high = 0xA1;  low = 0xDD; // '-'
        break;
      case 0x2E : high = 0xA1;  low = 0xA5; // '.'
        break;
      case 0x2F : high = 0xA1;  low = 0xBF; // '/'
        break;
      case 0x3A : high = 0xA1;  low = 0xA7; // ':'
        break;
      case 0x3B : high = 0xA1;  low = 0xA8; // ';'
        break;
      case 0x3C : high = 0xA1;  low = 0xE3; // '<'
        break;
      case 0x3D : high = 0xA1;  low = 0xE1; // '='
        break;
      case 0x3E : high = 0xA1;  low = 0xE4; // '>'
        break;
      case 0x3F : high = 0xA1;  low = 0xA9; // '?'
        break;
      case 0x40 : high = 0xA1;  low = 0xF7; // '@'
        break;
      case 0x5B : high = 0xA1;  low = 0xCE; // '['
        break;
      case 0x5C : high = 0xA1;  low = 0xEF; // '\'
        break;
      case 0x5D : high = 0xA1;  low = 0xCF; // ']'
        break;
      case 0x5E : high = 0xA1;  low = 0xB0; // '^'
        break;
      case 0x5F : high = 0xA1;  low = 0xB2; // '_'
        break;
      case 0x60 : high = 0xA1;  low = 0xAE; // '`'
        break;
      case 0x7B : high = 0xA1;  low = 0xD0; // '{'
        break;
      case 0x7C : high = 0xA1;  low = 0xC3; // '|'
        break;
      case 0x7D : high = 0xA1;  low = 0xD1; // '}'
        break;
      case 0x7E : high = 0xA1;  low = 0xB1; // '~'
        break;
      }
    }
  }
  else if (iseuchkana(high))    // EUC半角カタカナ
  {
    if (0xA7 <= low && low <= 0xAB) // ｧ-ｫ
    { high = 0xA5;  low = 0xA1 + (low - 0xA7) * 2; }
    else if (0xAC <= low && low <= 0xAE) // ｬｭｮ
    { high = 0xA5;  low = 0xE3 + (low - 0xAC) * 2; }
    else if (0xB1 <= low && low <= 0xB5) // ｱ-ｵ
    { high = 0xA5;  low = 0xA2 + (low - 0xB1) * 2; }
    else if (0xB6 <= low && low <= 0xC1) // ｶ-ﾁ
    { high = 0xA5;  low = 0xAB + (low - 0xB6) * 2; }
    else if (0xC5 <= low && low <= 0xC9) // ﾅ-ﾉ
    { high = 0xA5;  low = 0xCA + low - 0xC5; }
    else if (0xCA <= low && low <= 0xCE) // ﾊ-ﾎ
    { high = 0xA5;  low = 0xCF + (low - 0xCA) * 3; }
    else if (0xCF <= low && low <= 0xD0) // ﾏ-ﾐ
    { high = 0xA5;  low = 0xDE + low - 0xCF; }
    else if (0xD1 <= low && low <= 0xD3) // ﾑ-ﾓ
    { high = 0xA5;  low = 0xE0 + low - 0xD1; }
    else if (0xD4 <= low && low <= 0xD6) // ﾔﾕﾖ
    { high = 0xA5;  low = 0xE4 + (low - 0xD4) * 2; }
    else if (0xD7 <= low && low <= 0xDB) // ﾗ-ﾛ
    { high = 0xA5;  low = 0xE9 + low - 0xD7; }
    else
    {
      switch (low)
      {
      case 0xA1 : high = 0xA1;  low = 0xA3; // '｡'
        break;
      case 0xA2 : high = 0xA1;  low = 0xD6; // '｢'
        break;
      case 0xA3 : high = 0xA1;  low = 0xD7; // '｣'
        break;
      case 0xA4 : high = 0xA1;  low = 0xA2; // '､'
        break;
      case 0xA5 : high = 0xA1;  low = 0xA6; // '･'
        break;
      case 0xA6 : high = 0xA5;  low = 0xF2; // 'ｦ'
        break;
      case 0xAF : high = 0xA5;  low = 0xC3; // 'ｯ'
        break;
      case 0xB0 : high = 0xA1;  low = 0xBC; // 'ｰ'
        break;
      case 0xC2 : high = 0xA5;  low = 0xC4; // 'ﾂ'
        break;
      case 0xC3 : high = 0xA5;  low = 0xC6; // 'ﾃ'
        break;
      case 0xC4 : high = 0xA5;  low = 0xC8; // 'ﾄ'
        break;
      case 0xDC : high = 0xA5;  low = 0xEF; // 'ﾜ'
        break;
      case 0xDD : high = 0xA5;  low = 0xF3; // 'ﾝ'
        break;
      case 0xDE : high = 0xA1;  low = 0xAB; // 'ﾞ'
        break;
      case 0xDF : high = 0xA1;  low = 0xAC; // 'ﾟ'
        break;
      }
    }
  }
  return ctow(high, low);       // 変換できない文字はそのまま返す
}

wchar_t fun::euczentohan(uchar& high, uchar& low)
// EUC全角漢字を半角ANKにして返す
{
  switch (high)
  {
  case 0xA1 :
    switch (low)
    {
    case 0xA1 : high = 0x00;  low = 0x20; // "　"
      break;
    case 0xA2 : high = 0x8E;  low = 0xA4; // "、"
      break;
    case 0xA3 : high = 0x8E;  low = 0xA1; // "。"
      break;
    case 0xA4 : high = 0x00;  low = 0x2C; // "，"
      break;
    case 0xA5 : high = 0x00;  low = 0x2E; // "．"
      break;
    case 0xA6 : high = 0x8E;  low = 0xA5; // "・"
      break;
    case 0xA7 : high = 0x00;  low = 0x3A; // "："
      break;
    case 0xA8 : high = 0x00;  low = 0x3B; // "；"
      break;
    case 0xA9 : high = 0x00;  low = 0x3F; // "？"
      break;
    case 0xAA : high = 0x00;  low = 0x21; // "！"
      break;
    case 0xAB : high = 0x8E;  low = 0xDE; // "゛"
      break;
    case 0xAC : high = 0x8E;  low = 0xDF; // "゜"
      break;
    case 0xAE : high = 0x00;  low = 0x60; // "｀"
      break;
    case 0xB0 : high = 0x00;  low = 0x5E; // "＾"
      break;
    case 0xB1 : high = 0x00;  low = 0x7E; // "￣"
      break;
    case 0xB2 : high = 0x00;  low = 0x5F; // "＿"
      break;
    case 0xBC : high = 0x8E;  low = 0xB0; // "ー"
      break;
    case 0xBF : high = 0x00;  low = 0x2F; // "／"
      break;
    case 0xC3 : high = 0x00;  low = 0x7C; // "｜"
      break;
    case 0xCA : high = 0x00;  low = 0x28; // "（"
      break;
    case 0xCB : high = 0x00;  low = 0x29; // "）"
      break;
    case 0xCE : high = 0x00;  low = 0x5B; // "［"
      break;
    case 0xCF : high = 0x00;  low = 0x5D; // "」"
      break;
    case 0xD0 : high = 0x00;  low = 0x7B; // "｛"
      break;
    case 0xD1 : high = 0x00;  low = 0x7D; // "｝"
      break;
    case 0xD6 : high = 0x8E;  low = 0xA2; // "「"
      break;
    case 0xD7 : high = 0x8E;  low = 0xA3; // "」"
      break;
    case 0xDC : high = 0x00;  low = 0x2B; // "＋"
      break;
    case 0xDD : high = 0x00;  low = 0x2D; // "─"
      break;
    case 0xE1 : high = 0x00;  low = 0x3D; // "＝"
      break;
    case 0xE3 : high = 0x00;  low = 0x3C; // "＜"
      break;
    case 0xE4 : high = 0x00;  low = 0x3E; // "＞"
      break;
    case 0xEC : high = 0x00;  low = 0x27; // "’"
      break;
    case 0xED : high = 0x00;  low = 0x22; // "”"
      break;
    case 0xEF : high = 0x00;  low = 0x5C; // "￥"
      break;
    case 0xF0 : high = 0x00;  low = 0x24; // "＄"
      break;
    case 0xF3 : high = 0x00;  low = 0x25; // "％"
      break;
    case 0xF4 : high = 0x00;  low = 0x23; // "＃"
      break;
    case 0xF5 : high = 0x00;  low = 0x26; // "＆"
      break;
    case 0xF6 : high = 0x00;  low = 0x2A; // "＊"
      break;
    case 0xF7 : high = 0x00;  low = 0x40; // "＠"
      break;
    }
    break;
  case 0xA3 :
    if (0xB0 <= low && low <= 0xB9) // ０-９
    { high = 0x00;  low = low - 0x80; }
    else if (0xC1 <= low && low <= 0xDA) // Ａ-Ｚ
    { high = 0x00;  low = low - 0x80; }
    else if (0xE1 <= low && low <= 0xFA) // ａ-ｚ
    { high = 0x00;  low = low - 0x80; }
    break;
  case 0xA5 :
    if (0xCA <= low && low <= 0xCE) // ナ-ノ
    { high = 0x8E;  low = 0xC5 + low - 0xCA; }
    else if (0xDE <= low && low <= 0xE2) // マ-モ
    { high = 0x8E;  low = 0xCF + low - 0xDE; }
    else if (0xE9 <= low && low <= 0xED) // ラ-ロ
    { high = 0x8E;  low = 0xD7 + low - 0xE9; }
    else
    {
      switch (low)
      {
      case 0xA1 : high = 0x8E;  low = 0xA7; // "ァ"
        break;
      case 0xA2 : high = 0x8E;  low = 0xB1; // "ア"
        break;
      case 0xA3 : high = 0x8E;  low = 0xA8; // "ィ"
        break;
      case 0xA4 : high = 0x8E;  low = 0xB2; // "イ"
        break;
      case 0xA5 : high = 0x8E;  low = 0xA9; // "ゥ"
        break;
      case 0xA6 : high = 0x8E;  low = 0xB3; // "ウ"
        break;
      case 0xA7 : high = 0x8E;  low = 0xAA; // "ェ"
        break;
      case 0xA8 : high = 0x8E;  low = 0xB4; // "エ"
        break;
      case 0xA9 : high = 0x8E;  low = 0xAB; // "ォ"
        break;
      case 0xAA : high = 0x8E;  low = 0xB5; // "オ"
        break;
      case 0xAB : high = 0x8E;  low = 0xB6; // "カ"
        break;
      case 0xAD : high = 0x8E;  low = 0xB7; // "キ"
        break;
      case 0xAF : high = 0x8E;  low = 0xB8; // "ク"
        break;
      case 0xB1 : high = 0x8E;  low = 0xB9; // "ケ"
        break;
      case 0xB3 : high = 0x8E;  low = 0xBA; // "コ"
        break;
      case 0xB5 : high = 0x8E;  low = 0xBB; // "サ"
        break;
      case 0xB7 : high = 0x8E;  low = 0xBC; // "シ"
        break;
      case 0xB9 : high = 0x8E;  low = 0xBD; // "ス"
        break;
      case 0xBB : high = 0x8E;  low = 0xBE; // "セ"
        break;
      case 0xBD : high = 0x8E;  low = 0xBF; // "ソ"
        break;
      case 0xBF : high = 0x8E;  low = 0xC0; // "タ"
        break;
      case 0xC1 : high = 0x8E;  low = 0xC1; // "チ"
        break;
      case 0xC3 : high = 0x8E;  low = 0xAF; // "ッ"
        break;
      case 0xC4 : high = 0x8E;  low = 0xC2; // "ツ"
        break;
      case 0xC6 : high = 0x8E;  low = 0xC3; // "テ"
        break;
      case 0xC8 : high = 0x8E;  low = 0xC4; // "ト"
        break;
      case 0xCF : high = 0x8E;  low = 0xCA; // "ハ"
        break;
      case 0xD2 : high = 0x8E;  low = 0xCB; // "ヒ"
        break;
      case 0xD5 : high = 0x8E;  low = 0xCC; // "フ"
        break;
      case 0xD8 : high = 0x8E;  low = 0xCD; // "ヘ"
        break;
      case 0xDB : high = 0x8E;  low = 0xCE; // "ホ"
        break;
      case 0xE3 : high = 0x8E;  low = 0xAC; // "ャ"
        break;
      case 0xE4 : high = 0x8E;  low = 0xD4; // "ヤ"
        break;
      case 0xE5 : high = 0x8E;  low = 0xAD; // "ュ"
        break;
      case 0xE6 : high = 0x8E;  low = 0xD5; // "ユ"
        break;
      case 0xE7 : high = 0x8E;  low = 0xAE; // "ョ"
        break;
      case 0xE8 : high = 0x8E;  low = 0xD6; // "ヨ"
        break;
      case 0xEF : high = 0x8E;  low = 0xDC; // "ワ"
        break;
      case 0xF2 : high = 0x8E;  low = 0xA6; // "ヲ"
        break;
      case 0xF3 : high = 0x8E;  low = 0xDD; // "ン"
        break;
      }
    }
    break;
  }
  return ctow(high, low);       // 変換できない文字はそのまま返す
}
