// packcode.C  Version 2.20  Copyleft (c) 杉本典夫
// Time-stamp: <2022-11-17 17:08:07 tac>
//
// 可変長コード列をMBSFirstモードでバイト列にパックする共通関数
//
// 関連ライブラリ：libfun.so.2.2
// コンパイル法：g++ -O -fPIC -c packcode.C

#include <fun.H>                // 共通関数用ヘッダファイル

using namespace fun;            // fun名前空間使用(名前空間std含む)

namespace                       // 名前無し名前空間(局所定数用)
{
  constexpr int Tclear = 256;   // テーブルクリアコード(2の8乗,lzw用)
  constexpr int EOI = 257;      // End Of Information コード(2の8乗+1,lzw用)
}

int fun::packvcode(uint8* dest, uint c, int nbits, int cmd)
// 可変長コードをMBSFirstモードでバイトにパックする
{
  static int n = 0;
  static int shift = 8;
  if (cmd == Tclear)            // 初期設定コード
  {
    n = 0;  shift = 8;
    return 0;
  }
  else if (cmd == EOI)          // 終了処理
  {
    int ntmp = (shift == 8) ? n : n + 1;
    n = 0;  shift = 8;
    return ntmp;
  }
  if (shift == 8) dest[n] = 0;
  shift -= nbits;
  if (shift > 0)
  { dest[n] |= uint8(c << shift); }
  else if (shift == 0)
  {
    dest[n] |= uint8(c);
    ++n;  shift = 8;
  }
  else
  {
    do
    {
      shift = -shift;
      dest[n] |= uint8(c >> shift);
      ++n;  shift = 8 - shift;
      dest[n] = 0;
    } while (shift < 0);
    if (shift > 0)
    { dest[n] = uint8(c << shift); }
    else
    {
      dest[n] = uint8(c);
      ++n;  shift = 8;
    }
  }
  return n;                     // パック後のバイト数を返す
}

int fun::packcode(uint8* dest, const uint8* source, size_t ns, int nbits)
// 可変長コード列をMBSFirstモードでバイト列にパックする
{
  packvcode(dest, 0, nbits, Tclear);
  for (size_t i = 0; i < ns; ++i) packvcode(dest, source[i], nbits, 0);
  return packvcode(dest, 0, nbits, EOI); // パック後のバイト数を返す
}

int fun::unpackvcode(const uint8* source, size_t ns, int nbits, int cmd)
// MBSFirstモードでパックされた可変長コードをアンパックする
{
  static uint mask = 0, bits = 0, idx = 0;
  static int shift = 8;
  if (mask == 0 || cmd == Tclear) // 初期設定
  {
    mask = pow2(nbits) - 1;  bits = nbits;  idx = 0;  shift = 8;
    return 0;
  }
  if (idx >= ns)                // コード終了
  {
    mask = bits = idx = 0;  shift = 8;
    return -1;
  }
  if (uint(nbits) != bits) { mask = pow2(nbits) - 1;  bits = uint(nbits); }
  uint c = 0, ctmp = source[idx];
  shift -= nbits;
  if (shift > 0)
  { c = ctmp >> shift; }
  else if (shift == 0)
  {
    c = ctmp;
    ++idx;  shift = 8;
  }
  else
  {
    do
    {
      shift = -shift;
      c |= ctmp << shift;
      ++idx;  shift = 8 - shift;
      ctmp = source[idx];
    } while (shift < 0);
    if (shift > 0)
    { c |= ctmp >> shift; }
    else
    {
      c |= ctmp;
      ++idx;  shift = 8;
    }
  }
  return mask & c;              // アンパックしたコードを返す
}

int fun::unpackcode(uint8* dest, const uint8* source, size_t ns, int nbits)
// MBSFirstモードでパックされた可変長コード列をバイト列にアンパックする
{
  int n = 0, c = 0;
  unpackvcode(source, ns, nbits, Tclear);
  while ((c = unpackvcode(source, ns, nbits, 0)) >= 0)
  {
    dest[n] = uint8(c);
    ++n;
  }
  return n;                     // アンパック後のバイト数を返す
}
