// str.C  Version 2.20  Copyleft (c) 杉本典夫
// Time-stamp: <2022-11-17 17:14:29 tac>
//
// 文字列の各種処理を行う関数群(特殊文字と正規表現を含む文字列処理用関数も含む)
// 文字列の日本語コードはEUCまたはUTF8であること。
//
// 特殊文字指定法は次のとおり。
//   \2,\0:NUL(ヌル文字=0x00)   \A:SOH(0x01) \B:STX(0x02) \C:ETX(0x03)
//   \D:EOT(0x04)  \E:ENQ(0x05) \F:ACK(0x06) \G:BEL(0x07) \H,\b:BS(後退=0x08)
//   \I,\t:HT(水平タブ=0x09)    \J,\n:LF(改行=0x0A)    \K,\v:VT(垂直タブ=0x0B)
//   \L,\f:FF(改頁=0x0C)        \M,\r:CR(復帰=0x0D)       \N:SO(0x0E)
//   \O:SI(0x0F)   \P:DLE(0x10) \Q:DC1(0x11) \R:DC2(0x12) \S:DC3(0x13)
//   \T:DC4(0x14)  \U:NAK(0x15) \V:SYN(0x16) \W:ETB(0x17) \X:CAN(0x18)
//   \Y:EM(0x19)   \Z:SUB(0x1A) \e:ESC(エスケープ=0x1B)   \[:FS(0x1C)
//   \]:GS(0x1D)   \7:KS(0x1E)  \-:US(0x1F)
//   \x??:任意の16進数     \<ANK>:<ANK>文字自身(\,',"等)
//
// "で囲むと空白扱い文字と区切り文字と'の機能を抑制する。
//
// ライブラリ：libfun.so.2.0
// コンパイル法：g++ -O -fPIC -c str.C

#include <fun.H>                // 共通関数用ヘッダファイル

using namespace fun;            // fun名前空間使用(名前空間std含む)

namespace                       // 名前無し名前空間(局所関数定義用)
{

char sstr_c(const string& str)
// 特殊文字を通常文字に変換して返す(局所関数)
{
  if (str.empty()) return '\0';
  if (str[0] != CHResc) return str[0]; // 特殊文字以外は文字そのものを返す
  switch (str[1])
  {
  case '0' :
  case '2' : return 0x00;       // NUL(ヌル文字)
  case 'A' : return 0x01;       // SOH
  case 'B' : return 0x02;       // STX
  case 'C' : return 0x03;       // ETX
  case 'D' : return 0x04;       // EOT
  case 'E' : return 0x05;       // ENQ
  case 'F' : return 0x06;       // ACK
  case 'G' : return 0x07;       // BEL
  case 'b' :
  case 'H' : return 0x08;       // BS(後退)
  case 't' :
  case 'I' : return 0x09;       // HT(水平タブ)
  case 'n' :
  case 'J' : return 0x0A;       // LF(改行)
  case 'v' :
  case 'K' : return 0x0B;       // VT(垂直タブ)
  case 'f' :
  case 'L' : return 0x0C;       // FF(改頁)
  case 'r' :
  case 'M' : return 0x0D;       // CR(復帰)
  case 'N' : return 0x0E;       // SO
  case 'O' : return 0x0F;       // SI
  case 'P' : return 0x10;       // DLE
  case 'Q' : return 0x11;       // DC1
  case 'R' : return 0x12;       // DC2
  case 'S' : return 0x13;       // DC3
  case 'T' : return 0x14;       // DC4
  case 'U' : return 0x15;       // NAK
  case 'V' : return 0x16;       // SYN
  case 'W' : return 0x17;       // ETB
  case 'X' : return 0x18;       // CAN
  case 'Y' : return 0x19;       // EM
  case 'Z' : return 0x1A;       // SUB
  case 'e' : return 0x1B;       // ESC(エスケープ)
  case '[' : return 0x1C;       // FS
  case ']' : return 0x1D;       // GS
  case '7' : return 0x1E;       // KS
  case '-' : return 0x1F;       // US
  case 'x' : if (str.size() == 2) return str[1]; // 'x'を返す
    return char(strtoul(str.c_str() + 2, 0, 16)); // 16進数表現文字
  }
  return str[1];                // その他の文字は文字そのものを返す
}

string& c_sstr(uchar sc)
// 文字を特殊文字列に変換する(局所関数)
{
  static string str;
  str = CHResc;
  switch (sc)
  {
  case 0x00 : str += '0';  break;
  case 0x01 : str += 'A';  break;
  case 0x02 : str += 'B';  break;
  case 0x03 : str += 'C';  break;
  case 0x04 : str += 'D';  break;
  case 0x05 : str += 'E';  break;
  case 0x06 : str += 'F';  break;
  case 0x07 : str += 'G';  break;
  case 0x08 : str += 'b';  break;
  case 0x09 : str += 't';  break;
  case 0x0A : str += 'n';  break;
  case 0x0B : str += 'v';  break;
  case 0x0C : str += 'f';  break;
  case 0x0D : str += 'r';  break;
  case 0x0E : str += 'N';  break;
  case 0x0F : str += 'O';  break;
  case 0x10 : str += 'P';  break;
  case 0x11 : str += 'Q';  break;
  case 0x12 : str += 'R';  break;
  case 0x13 : str += 'S';  break;
  case 0x14 : str += 'T';  break;
  case 0x15 : str += 'U';  break;
  case 0x16 : str += 'V';  break;
  case 0x17 : str += 'W';  break;
  case 0x18 : str += 'X';  break;
  case 0x19 : str += 'Y';  break;
  case 0x1A : str += 'Z';  break;
  case 0x1B : str += 'e';  break;
  case 0x1C : str += '[';  break;
  case 0x1D : str += ']';  break;
  case 0x1E : str += '7';  break;
  case 0x1F : str += '-';  break;
  case CHRsquot :
  case CHRdquot :
  case CHResc : str += sc;  break;
  default : str = sc;
  }
  return str;                   // 変換後の文字列を返す
}

int euc_sstrtostr(string& str)
// 特殊文字列を通常文字列に変換する(局所関数)
{
  if (str.empty()) return 0;    // ヌル文字列
  string outstr, buf;
  bool dquot = false;
  int escc = 0;
  uchar cpre = '\0';
  strsize_t i = 0, n = str.size();
  while (i < n)
  {
    uchar c = str[i];
    if (escc > 2)               // 特殊文字３文字目
    {
      if (isxdigit(c))          // 16進数文字
      {
        buf += c;
        outstr += sstr_c(buf);
        escc = 0;  cpre = '\0';
      }
      else
      {
        cpre = buf[2];
        buf.erase(2);
        outstr += sstr_c(buf);
        outstr += cpre;
        if (c == CHResc) { escc = 1;  buf = cpre = c; }
        else { outstr += c;  escc = 0;  cpre = '\0'; }
      }
    }
    else if (escc == 2)         // 特殊文字２文字目
    {
      if (isxdigit(c))          // 16進数文字
      { ++escc;  buf += c; }
      else
      {
        outstr += sstr_c(buf);
        if (c == CHResc) { escc = 1;  buf = cpre = c; }
        else { outstr += c;  escc = 0;  cpre = '\0'; }
      }
    }
    else if (escc == 1)         // 特殊文字１文字目
    {
      ++escc;  buf += c;
      if (c != 'x') { outstr += sstr_c(buf);  escc = 0;  cpre = '\0'; }
    }
    else if (c == CHResc)       // '\'で始まる特殊文字
    { escc = 1;  buf = cpre = c; }
    else if (dquot)             // 復引用文字列中、'を無視
    {
      if (c == CHRdquot) dquot = false; // 復引用文字列終了
      else outstr += c;
    }
    else if (c == CHRdquot)     // 復引用文字「"」
    { dquot = true; }
    else                        // その他のANK文字
    { outstr += c; }
    ++i;
  }
  str = outstr;
  return int(str.size());       // 変換後の文字数を返す
}

} // 名前無し名前空間終わり

int fun::strchomp(string& str)
// 文字列の末尾の改行記号を削除する(汎用)
{
  strsize_t n = str.size();
  if (n < 1) return 0;          // 無変換
  if (str[n - 1] == '\r')
  {
    str.resize(n - 1);
    --n;
  }
  if (n < 1) return 0;
  if (str[n - 1] == '\n') str.resize(n - 1);
  return int(n - 1);            // 変換後の文字数を返す
}

int fun::strunquot(string& str, codetype cdtype)
// 文字列を囲む'または"を削除する(汎用)
{
  if (str.size() < 2) return 0;
  if (cdtype == codenon) cdtype = langcode(); // コードが指定されていない時
  int rconv = 0;
  switch (cdtype)               // 変換用にEUCにする
  {
  case jis : rconv = strjistoeuc(str);
    break;
  case sjis : rconv = strsjistoeuc(str);
  }
  if (str[0] == CHRsquot && str[str.size() - 1] == CHRsquot)
  {                             // 最初と最後の'だけ削除する
    str.erase(0, 1);
    str.resize(str.size() - 1);
  }
  else if (str[0] == CHRdquot && str[str.size() - 1] == CHRdquot)
  {                             // 最初と最後の"だけ削除する
    str.erase(0, 1);
    str.resize(str.size() - 1);
  }
  if (rconv < 0) return rconv;  // コード変換エラー時は元のまま変換する
  switch (cdtype)               // EUCから元のコードに戻す
  {
  case jis : rconv = streuctojis(str);
    break;
  case sjis : rconv = streuctosjis(str);
  }
  if (rconv < 0) return rconv;  // 変換エラー
  return int(str.size());       // 変換後の文字数を返す
}

int fun::strlower(string& str, codetype cdtype)
// 文字列中のアルファベットを小文字にする(汎用)
{
  if (str.empty()) return 0;    // ヌル文字列
  if (cdtype == codenon) cdtype = langcode(); // コードが指定されていない時
  int rconv = 0;
  switch (cdtype)               // 変換用にEUCにする
  {
  case jis : rconv = strjistoeuc(str);
    break;
  case sjis : rconv = strsjistoeuc(str);
  }
  for (strsize_t i = 0; i < str.size(); ++i)
  {
    uchar c = str[i];
    if (isalpha(c))             // 1バイトのアルファベットだけ変換
    {
      char& ci = str[i];        // 文字列中のｉ番目の文字の参照
      ci = tolower(c);
    }
  }
  if (rconv < 0) return rconv;  // コード変換エラー時は元のまま変換する
  switch (cdtype)               // EUCから元のコードに戻す
  {
  case jis : rconv = streuctojis(str);
    break;
  case sjis : rconv = streuctosjis(str);
  }
  if (rconv < 0) return rconv;  // 変換エラー
  return int(str.size());       // 変換後の文字数を返す
}

int fun::strupper(string& str, codetype cdtype)
// 文字列中のアルファベットを大文字にする(汎用)
{
  if (str.empty()) return 0;    // ヌル文字列
  if (cdtype == codenon) cdtype = langcode(); // コードが指定されていない時
  int rconv = 0;
  switch (cdtype)               // 変換用にEUCにする
  {
  case jis : rconv = strjistoeuc(str);
    break;
  case sjis : rconv = strsjistoeuc(str);
  }
  for (strsize_t i = 0; i < str.size(); ++i)
  {
    uchar c = str[i];
    if (isalpha(c))             // １バイトのアルファベットだけ変換
    {
      char& ci = str[i];        // 文字列中のｉ番目の文字の参照
      ci = toupper(c);
    }
  }
  if (rconv < 0) return rconv;  // コード変換エラー時は元のまま変換する
  switch (cdtype)               // EUCから元のコードに戻す
  {
  case jis : rconv = streuctojis(str);
    break;
  case sjis : rconv = streuctosjis(str);
  }
  if (rconv < 0) return rconv;  // 変換エラー
  return int(str.size());       // 変換後の文字数を返す
}

int fun::strreplace(string& str, const string& targstr, const string& reprstr, bool global, codetype cdtype)
// 文字列中の部分文字列を置換する(汎用)
{
  if (str.empty()) return 0;    // 置換なし
  if (cdtype == codenon) cdtype = langcode(); // コードが指定されていない時
  string ttargstr(targstr), treprstr(reprstr);
  int rconv = 0;
  switch (cdtype)               // 変換用にUTF-8にする
  {
  case euc : rconv = streuctoutf8(str);
    if (rconv >= 0) rconv = streuctoutf8(ttargstr);
    if (rconv >= 0) rconv = streuctoutf8(treprstr);
    break;
  case jis : rconv = strjistoutf8(str);
    if (rconv >= 0) rconv = strjistoutf8(ttargstr);
    if (rconv >= 0) rconv = strjistoutf8(treprstr);
    break;
  case sjis : rconv = strsjistoutf8(str);
    if (rconv >= 0) rconv = strsjistoutf8(ttargstr);
    if (rconv >= 0) rconv = strsjistoutf8(treprstr);
  }
  strsize_t idx = str.find(ttargstr);
  if (idx == string::npos) return 0;
  str.replace(idx, ttargstr.size(), treprstr);
  int nrep = 1;
  if (global)                   // 全て置換する場合
  {
    while ((idx = str.find(ttargstr, idx + treprstr.size())) != string::npos)
    { str.replace(idx, ttargstr.size(), treprstr);  ++nrep; }
  }
  if (rconv < 0) return rconv;  // コード変換エラー時は元のまま変換する
  switch (cdtype)               // UTF-8から元のコードに戻す
  {
  case euc : rconv = strutf8toeuc(str);
    break;
  case jis : rconv = strutf8tojis(str);
    break;
  case sjis : rconv = strutf8tosjis(str);
  }
  if (rconv < 0) return rconv;  // 変換エラー
  return nrep;                  // 置換数を返す
}

int fun::strsplit(lstr& dstrlst, const string& sstr, char delmc, codetype cdtype)
// 文字列を区切り文字で分割して文字列リストに追加する(汎用)
{
  if (sstr.empty()) return 0;   // 文字列が空の場合
  if (cdtype == codenon) cdtype = langcode(); // コードが指定されていない時
  string tstr(sstr), substr;
  if (delmc == ' ') strtrm(tstr);
  size_t size0 = dstrlst.size();
  strsize_t idx = string::npos;
  while ((idx = getsubstr(substr, tstr, delmc, cdtype)) != string::npos)
  {
    dstrlst.push_back(substr);
    if (idx < tstr.size()) tstr.erase(0, idx + 1);
    else break;                 // 文字列終了
    if (delmc == ' ') strtrm(tstr);
  }
  if (idx == string::npos) dstrlst.push_back(""); // 最後に区切り文字がある場合
  return dstrlst.size() - size0; // 追加した要素数を返す
}

int fun::strsplit(vecstr& dstrvec, const string& sstr, char delmc, codetype cdtype)
// 文字列を区切り文字で分割して文字列ベクトルに追加する(汎用)
{
  lstr dstrlst;
  if (cdtype == codenon) cdtype = langcode(); // コードが指定されていない時
  if (strsplit(dstrlst, sstr, delmc, cdtype) < 1) return 0; // 文字列取得できず
  size_t size0 = dstrvec.size();
  dstrvec.resize(size0 + dstrlst.size()); // ベクトルのサイズを増やす
  int i = size0;
  for (lstr_itr pos = dstrlst.begin(); pos != dstrlst.end(); ++pos, ++i)
  { dstrvec[i] = *pos; }
  return dstrvec.size() - size0; // 追加したサイズを返す
}

strsize_t fun::getsubstr(string& substr, const string& sstr, char delmc, codetype cdtype)
// 文字列から区切り文字までの部分文字列を取り出す(汎用)
// 部分文字列の中に区切り記号を含む場合は
// 復引用文字「"」で囲むか「\記号」という形式で表す。
// それらの特殊表現文字を普通の文字に変換するにはsstrtostr()関数を用いる
{
  if (sstr.empty()) return string::npos;
  if (cdtype == codenon) cdtype = langcode(); // コードが指定されていない時
  string tstr(sstr);
  int rconv = 0;
  switch (cdtype)               // 変換用にEUCにする
  {
  case jis : rconv = strjistoeuc(tstr);
    break;
  case sjis : rconv = strsjistoeuc(tstr);
  }
  substr.erase();
  bool dquot = false;
  uchar cpre = '\0';
  strsize_t i = 0;
  if (delmc == ' ') while (i < tstr.size() && tstr[i] == delmc) ++i;
  while (i < tstr.size())
  {
    uchar c = tstr[i];
    if (cpre == CHResc)         // \<文字>
    { substr += c;  cpre = '\0'; }
    else if (c == CHResc)       // エスケープ文字「\」
    { substr += c;  cpre = c; }
    else if (dquot)             // 復引用文字列中
    {
      substr += c;
      if (c == CHRdquot) dquot = false; // 復引用文字列終了
    }
    else if (c == CHRdquot)     // 復引用文字「"」
    { substr += c;  dquot = true; }
    else if (c == delmc)        // 区切り文字delmc
    { break; }
    else                        // その他のANK
    { substr += c; }
    ++i;
  }
  if (rconv < 0) return rconv;  // コード変換エラー時は元のまま変換する
  switch (cdtype)               // EUCから元のコードに戻す
  {
  case jis : rconv = streuctojis(substr);
    break;
  case sjis : rconv = streuctosjis(substr);
  }
  if (rconv < 0) return rconv;  // 変換エラー
  return i;                     // 区切り記号のオフセットを返す
}

char* fun::getsubstr(string& substr, const char* sstr, char delmc, codetype cdtype)
// 文字配列から区切り文字までの部分文字列を取り出す(EUC/UTF-8/Shift-JISのみ)
// 部分文字列の中に区切り記号を含む場合は
// 復引用文字「"」で囲むか「\記号」という形式で表す。
// それらの特殊表現文字を普通の文字に変換するにはsstrtostr()関数を用いる
// 戻り値　nullptr:空文字列の場合、'\0'へのポインター:'\0'で終わる文字列の場合
//       区切り文字へのポインター:文字列の途中の場合
// 呼出側はこれらの戻り値によって適当な処理をする。
{
  if (sstr == nullptr || *sstr == '\0') return nullptr; // 入力文字配列終了
  substr.erase();
  int nc = 0;
  string buf;
  bool dquot = false;
  char cpre = '\0';
  char* sptr = (char*)sstr;
  if (delmc == ' ') while (*sptr != '\0' && *sptr == delmc) ++sptr;
  while (*sptr != '\0')
  {
    char c = *sptr;
    if (nc > 1)                 // マルチバイトコードの途中
    { buf += c;  --nc; }
    else if (nc == 1)           // マルチバイトコードの最後
    {
      buf += c;
      substr += buf;
      buf.erase();  nc = 0;
    }
    else if (ismcode(c, cdtype)) // マルチバイトコード1バイト目
    { buf = c;  nc = ncode(c, cdtype) - 1; }
    else if (cpre == CHResc)    // \<文字>
    { substr += c;  cpre = '\0'; }
    else if (c == CHResc)       // エスケープ文字「\」
    { substr += c;  cpre = c; }
    else if (dquot)             // 復引用文字列中
    {
      substr += c;
      if (c == CHRdquot) dquot = false; // 復引用文字列終了
    }
    else if (c == CHRdquot)     // 復引用文字「"」
    { substr += c;  dquot = true; }
    else if (c == delmc)        // 区切り文字delmc
    { break; }
    else                        // その他のANK
    { substr += c; }
    ++sptr;
  }
  return sptr;                  // 区切り記号またはヌル文字へのポインターを返す
}

int fun::sstrtostr(string& str, codetype cdtype)
// 特殊文字列を通常文字列に変換する(汎用)
{
  if (str.empty()) return 0;    // ヌル文字列
  if (cdtype == codenon) cdtype = langcode(); // コードが指定されていない時
  int rconv = 0;
  switch (cdtype)               // 変換用にEUCにする
  {
  case jis : rconv = strjistoeuc(str);
    break;
  case sjis : rconv = strsjistoeuc(str);
  }
  euc_sstrtostr(str);
  if (rconv < 0) return rconv;  // コード変換エラー時は元のまま変換する
  switch (cdtype)               // EUCから元のコードに戻す
  {
  case jis : rconv = streuctojis(str);
    break;
  case sjis : rconv = streuctosjis(str);
  }
  if (rconv < 0) return rconv;  // 変換エラー
  return int(str.size());       // 変換後の文字数を返す
}

int fun::strtosstr(string& str)
// 文字列を特殊文字列に変換する(汎用)
{
  if (str.empty()) return 0;    // ヌル文字列
  string tstr;
  uchar cpre = '\0';
  for (strsize_t i = 0; i < str.size(); ++i) tstr += c_sstr(str[i]);
  str = tstr;
  return int(str.size());       // 変換後の文字数を返す
}

int fun::strpmatch(const string& str, const string& pattern, codetype cdtype)
// 文字列中にパターン(正規表現を含む)があるかどうかをチェックする(汎用)
{
  if (cdtype == codenon) cdtype = langcode(); // コードが指定されていない時
  string tstr(str), tpattern(pattern);
  int rconv = 0;
  switch (cdtype)               // 変換用にUTF-8にする(変換エラー時は元のまま処理する)
  {
  case euc : rconv = streuctoutf8(tstr);
    if (rconv >= 0) rconv = streuctoutf8(tpattern);
    break;
  case jis : rconv = strjistoutf8(tstr);
    if (rconv >= 0) rconv = strjistoutf8(tpattern);
    break;
  case sjis : rconv = strsjistoutf8(tstr);
    if (rconv >= 0) rconv = strsjistoutf8(tpattern);
  }
  regex_t preg;
  int err = regcomp(&preg, tpattern.c_str(), REG_EXTENDED);
  // 正規表現パターンをコンパイルする(拡張正規表現)
  constexpr size_t ERRbufsize = 80;
  char errbuf[ERRbufsize];
  if (err)
  {
    regerror(err, &preg, errbuf, ERRbufsize);
    cout << "regcomp:" << errbuf << "->" << pattern << endl;
    regfree(&preg);
    return -1;
  }
  constexpr size_t Nmatch = 2;
  regmatch_t pmatch[Nmatch];
  err = regexec(&preg, tstr.c_str(), Nmatch, pmatch, 0);
  if (err == REG_NOMATCH)       // パターンなし
  {
    regfree(&preg);
    return 0;
  }
  if (err)
  {
    regerror(err, &preg, errbuf, ERRbufsize);
    cout << "regexec:" << errbuf << "->" << pattern << endl;
    regfree(&preg);
    return -2;
  }
  regfree(&preg);
  return 1;                     // パターンあり
}

int fun::strregrep(string& sline, const string& pattern, const string& reprstr, bool global, codetype cdtype)
// 正規表現を含む部分文字列を置換する(汎用)
{
  if (cdtype == codenon) cdtype = langcode(); // コードが指定されていない時
  string tsline(sline), tpattern(pattern), treprstr(reprstr);
  int rconv = 0;
  switch (cdtype)               // 変換用にUTF-8にする
  {
  case euc : rconv = streuctoutf8(tsline);
    if (rconv >= 0) rconv = streuctoutf8(tpattern);
    if (rconv >= 0) rconv = streuctoutf8(treprstr);
    break;
  case jis : rconv = strjistoutf8(tsline);
    if (rconv >= 0) rconv = strjistoutf8(tpattern);
    if (rconv >= 0) rconv = strjistoutf8(treprstr);
    break;
  case sjis : rconv = strsjistoutf8(tsline);
    if (rconv >= 0) rconv = strsjistoutf8(tpattern);
    if (rconv >= 0) rconv = strsjistoutf8(treprstr);
  }
  regex_t preg;
  int err = regcomp(&preg, tpattern.c_str(), REG_EXTENDED);
  // 正規表現パターンをコンパイルする(拡張正規表現)
  constexpr size_t ERRbufsize = 80;
  char errbuf[ERRbufsize];
  if (err)
  {
    regerror(err, &preg, errbuf, ERRbufsize);
    cout << "regcomp:" << errbuf << "->" << pattern << endl;
    regfree(&preg);
    return -1;
  }
  constexpr size_t Nmatch = 2;
  regmatch_t pmatch[Nmatch];
  err = regexec(&preg, tsline.c_str(), Nmatch, pmatch, 0);
  if (err == REG_NOMATCH)       // パターンなし
  {
    regfree(&preg);
    return 0;
  }
  if (err)
  {
    regerror(err, &preg, errbuf, ERRbufsize);
    cout << "regexec:" << errbuf << "->" << pattern << endl;
    regfree(&preg);
    return -2;
  }
  tsline.replace(pmatch[0].rm_so, pmatch[0].rm_eo - pmatch[0].rm_so, treprstr);
  int nrep = 1;
  if (global && tpattern[0] != '^' && tpattern[tpattern.size() - 1] != '$')
  {
    while ((err = regexec(&preg, tsline.c_str(), Nmatch, pmatch, 0)) == 0)
    {
      tsline.replace(pmatch[0].rm_so, pmatch[0].rm_eo - pmatch[0].rm_so, treprstr);
      ++nrep;
    }
  }
  regfree(&preg);
  if (rconv < 0)               // コード変換エラー時は元のまま変換する
  {
    sline = tsline;
    return rconv;
  }
  switch (cdtype)               // UTF-8から元のコードに戻す
  {
  case euc : rconv = strutf8toeuc(tsline);
    break;
  case jis : rconv = strutf8tojis(tsline);
    break;
  case sjis : rconv = strutf8tosjis(tsline);
  }
  if (rconv < 0) return rconv;  // 変換エラー
  sline = tsline;
  return nrep;                  // 置換数を返す
}
